<?php

namespace App\Http\Controllers\Api\Admin;

use App\Http\Controllers\Controller;
use App\Library\FoxPaginate\TablePaginator;
use App\Library\FoxSocket\FoxSocket;
use App\Library\FoxTg\FoxTgApi;
use App\Models\Bot;
use GuzzleHttp\Exception\GuzzleException;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\MessageBag;

class BotController extends Controller
{
    public function index(Request $request): JsonResponse
    {
        return response()->json([
            'ok' => true,
            'data' => is_null($request->get('t')) ? TablePaginator::paginate(Bot::query(), [
                'per_page' => 10,
                'search_fields' => [
                    'phone',
                    'fullname',
                ]
            ]) : Bot::all()
        ]);
    }

    public function authorize(Request $request): JsonResponse
    {
        $needles = ['phone'];
        $errors = Validator::make($request->only($needles), [
            'phone' => 'required'
        ], [
            'phone.*' => 'شماره ربات را وارد کنید.'
        ])->errors();


        if ($errors->isEmpty()) {
            $tg_helper = new FoxTgApi(env('BOT_API'));
            try {
                $result = $tg_helper->call([
                    'method' => 'post',
                    'action' => 'authorize',
                    'data' => $request->only($needles)
                ]);
                return response()->json($result->response(), $result->status());
            } catch (GuzzleException $e) {
                return response()->json([
                    'ok' => false,
                    'errors' => [
                        'connection' => [
                            'CONNECTION_FAILED'
                        ]]
                ], $e->getCode());
            }
        } else {
            return response()->json([
                'ok' => false,
                'errors' => $errors
            ], 400);
        }
    }

    public function confirm(Request $request): JsonResponse
    {
        $needles = ['phone', 'token', 'code'];
        $errors = Validator::make($request->only($needles), [
            'phone' => 'required',
            'token' => 'required',
            'code' => 'required',
        ], [
            'phone.*' => 'شماره ربات را وارد کنید.',
            'token.*' => 'کلید احراز را وارد کنید.',
            'code.*' => 'کد تایید ربات را وارد کنید.',
        ])->errors();


        if ($errors->isEmpty()) {
            $tg_helper = new FoxTgApi(env('BOT_API'));
            try {
                $result = $tg_helper->call([
                    'method' => 'post',
                    'action' => 'confirm',
                    'data' => $request->only($needles)
                ]);
                if ($result->status() === 200) {
                    $fullname = $result->response()['data']['first_name'];
                    if ($result->response()['data']['last_name'])
                        $fullname = "{$fullname} {$result->response()['data']['last_name']}";
                    Bot::query()->updateOrCreate([
                        'phone' => $request->post('phone'),
                    ], [
                        'fullname' => $fullname,
                        'session_key' => $result->response()['data']['session'],
                    ]);
                    try {
                        $client = new FoxSocket();
                        $client->emit('refresh_bots');
                    } catch (\Exception $exception) {
                        error_log(print_r($exception, true));
                    }
                }
                return response()->json($result->response(), $result->status());
            } catch (GuzzleException $e) {
                return response()->json([
                    'ok' => false,
                    'errors' => [
                        'connection' => [
                            'CONNECTION_FAILED'
                        ]]
                ], $e->getCode());
            }
        } else {
            return response()->json([
                'ok' => false,
                'errors' => $errors
            ], 400);
        }
    }

    public function destroy(Bot $bot): JsonResponse
    {
        if ($bot->delete()) {
            try {
                $client = new FoxSocket();
                $client->emit('refresh_bots');
            } catch (\Exception $exception) {
                error_log(print_r($exception, true));
            }
            return response()->json([
                'ok' => true
            ]);
        } else
            return response()->json([
                'ok' => false,
                'errors' => new MessageBag([
                    'unknown' => ['خطا در عملیات!']
                ])
            ], 500);
    }

}
