<?php

namespace App\Http\Controllers\Api\Customer;

use App\Http\Controllers\Controller;
use App\Library\FoxPaginate\TablePaginator;
use App\Models\Operation;
use App\Models\User;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

class OperationController extends Controller
{
    public function index(): JsonResponse
    {
        /** @var User $user */
        $user = auth()->user();
        return response()->json([
            'ok' => true,
            'data' => TablePaginator::paginate($user->operations()->with(['bot', 'src', 'dest', 'archive']), [
                'per_page' => 10,
                'search_fields' => [
                    'title'
                ]
            ])
        ]);
    }


    public function store(Request $request): JsonResponse
    {
        $needles = ['bot_id', 'src_id', 'dest_id', 'archive_id', 'title', 'rules', 'enabled'];
        $errors = Validator::make($request->only($needles), [
            'bot_id' => 'required|exists:bots,_id',
            'src_id' => 'required|exists:channels,_id',
            'dest_id' => 'required|exists:channels,_id',
            'archive_id' => 'required|exists:channels,_id',
            'title' => 'required|string',
            'rules' => 'array',
            'enabled' => 'required|boolean',
        ], [
            'bot_id.*' => 'ربات انتخابی یافت نشد.',
            'src_id.*' => 'کانال مبدا صحیح نیست.',
            'dest_id.*' => 'کانال مقصد صحیح نیست.',
            'archive_id.*' => 'کانال آرشیو صحیح نیست.',
            'title.*' => 'عنوان عملیات صحیح نیست.',
            'rules.*' => 'قوانین به درستی وارد نشده.',
            'enabled.*' => 'لطفا وضعیت ربات را فعال یا غیرفعال کنید.',
        ])->errors();

        if ($errors->isEmpty()) {
            $customer = auth()->user();
            $operation = Operation::query()->create([
                'user_id' => $customer['_id'],
                'bot_id' => $request->post('bot_id'),
                'src_id' => $request->post('src_id'),
                'dest_id' => $request->post('dest_id'),
                'archive_id' => $request->post('archive_id'),
                'title' => $request->post('title'),
                'rules' => $request->post('rules'),
                'enabled' => $request->post('enabled')
            ]);

            return response()->json([
                'ok' => true,
                'data' => $operation['_id']
            ]);
        } else
            return response()->json([
                'ok' => false,
                'errors' => $errors
            ], 400);
    }

    public function update(Request $request, Operation $operation): JsonResponse
    {
        $needles = ['bot_id', 'src_id', 'dest_id', 'archive_id', 'title', 'rules', 'enabled'];
        $errors = Validator::make($request->only($needles), [
            'bot_id' => 'required|exists:bots,_id',
            'src_id' => 'required|exists:channels,_id',
            'dest_id' => 'required|exists:channels,_id',
            'archive_id' => 'required|exists:channels,_id',
            'title' => 'required|string',
            'rules' => 'array',
            'enabled' => 'required|boolean',
        ], [
            'bot_id.*' => 'ربات انتخابی یافت نشد.',
            'src_id.*' => 'کانال مبدا صحیح نیست.',
            'dest_id.*' => 'کانال مقصد صحیح نیست.',
            'archive_id.*' => 'کانال آرشیو صحیح نیست.',
            'title.*' => 'عنوان عملیات صحیح نیست.',
            'rules.*' => 'قوانین به درستی وارد نشده.',
            'enabled.*' => 'لطفا وضعیت ربات را فعال یا غیرفعال کنید.',
        ])->errors();

        if ($errors->isEmpty()) {
            $operation->update([
                'bot_id' => $request->post('bot_id'),
                'src_id' => $request->post('src_id'),
                'dest_id' => $request->post('dest_id'),
                'archive_id' => $request->post('archive_id'),
                'title' => $request->post('title'),
                'rules' => $request->post('rules'),
                'enabled' => $request->post('enabled')
            ]);

            return response()->json([
                'ok' => true,
            ]);
        } else
            return response()->json([
                'ok' => false,
                'errors' => $errors
            ], 400);
    }

    public function destroy(Operation $operation): JsonResponse
    {
        $operation->delete();
        return response()->json([
            'ok' => true
        ]);
    }
}
