<?php

namespace App\Library\FoxTg;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\Exception\RequestException;
use Illuminate\Support\MessageBag;

class FoxTgApi
{
    private string $api;
    private Client $client;

    public function __construct($api_url)
    {
        $this->api = $api_url;
        $this->client = new Client();
    }

    /**
     * @throws GuzzleException
     */
    public function call(mixed $attributes): TgResponse
    {
        $action = $attributes['action'];
        $method = $attributes['method'];
        $data = $attributes['data'];
        if (!empty($data))
            $data = [
                'form_params' => $data
            ];

        try {
            $result = $this->client->request($method, "{$this->api}/$action", $data);
            $body = @json_decode($result->getBody(), true);
            return new TgResponse($result->getStatusCode(), $body);
        } catch (RequestException|GuzzleException $e) {
            if ($e->hasResponse() && $e->getResponse()->getStatusCode() === 400) {
                $body = @json_decode($e->getResponse()->getBody(), true);
                return new TgResponse(400, [
                    'ok' => false,
                    'errors' => new MessageBag(['input' => [$body['error']['errorMessage']]])
                ]);
            } else
                throw $e;
        }
    }
}
