<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Jenssegers\Mongodb\Eloquent\Model;
use Jenssegers\Mongodb\Relations\BelongsToMany;
use Jenssegers\Mongodb\Relations\HasMany;

class Bot extends Model
{
    use HasFactory;

    protected $fillable = [
        'phone',
        'fullname',
        'session_key',
    ];

    protected $hidden = [
        'session_key',
        'user_ids',
        'channel_ids',
        'operation_ids',
    ];

    protected static function boot()
    {
        parent::boot();
        self::deleting(function (Bot &$model) {
            $model->users()->sync([]);
            $model->channels()->sync([]);
            $model->operations()->delete();
        });
    }

    public function users(): BelongsToMany
    {
        return $this->belongsToMany(User::class);
    }

    public function channels(): BelongsToMany
    {
        return $this->belongsToMany(Channel::class);
    }

    public function operations(): HasMany
    {
        return $this->hasMany(Operation::class);
    }
}
