<?php

namespace App\Models;

use App\Extentions\Traits\JwtTokenize;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Support\Str;
use Jenssegers\Mongodb\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Jenssegers\Mongodb\Relations\BelongsToMany;
use Jenssegers\Mongodb\Relations\HasMany;
use Laravel\Sanctum\HasApiTokens;
use Maklad\Permission\Traits\HasRoles;

class User extends Authenticatable
{
    use HasApiTokens, HasFactory, Notifiable, JwtTokenize, HasRoles;

    protected $collection = 'users';

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'token',
        'fullname',
        'username',
        'password',
        'max_channels',
    ];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    protected static function boot()
    {
        parent::boot();
        self::creating(function (User &$model) {
            $model['token'] = Str::random(80);
        });
        self::deleting(function (User $model) {
            $model->bots()->sync([]);
            $model->channels()->sync([]);
            $model->operations()->delete();
        });
    }

    public function getRemainWorkersAttribute()
    {
        return $this['max_channels'] - $this->channels()->count();
    }

    public function bots(): BelongsToMany
    {
        return $this->belongsToMany(Bot::class);
    }

    public function channels(): BelongsToMany
    {
        return $this->belongsToMany(Channel::class);
    }

    public function operations(): HasMany
    {
        return $this->hasMany(Operation::class);
    }
}
